# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('hcore', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SynopticGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.SlugField(unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='SynopticGroupStation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('station', models.ForeignKey(to='hcore.Station')),
                ('synoptic_group', models.ForeignKey(to='enhydris_synoptic.SynopticGroup')),
            ],
        ),
        migrations.CreateModel(
            name='SynopticTimeseries',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('order', models.PositiveSmallIntegerField()),
                ('synoptic_group_station', models.ForeignKey(to='enhydris_synoptic.SynopticGroupStation')),
                ('timeseries', models.ForeignKey(to='hcore.Timeseries')),
            ],
            options={
                'verbose_name_plural': 'Synoptic timeseries',
            },
        ),
        migrations.AddField(
            model_name='synopticgroupstation',
            name='timeseries',
            field=models.ManyToManyField(to='hcore.Timeseries', through='enhydris_synoptic.SynopticTimeseries'),
        ),
        migrations.AddField(
            model_name='synopticgroup',
            name='stations',
            field=models.ManyToManyField(to='hcore.Station', through='enhydris_synoptic.SynopticGroupStation'),
        ),
        migrations.AlterUniqueTogether(
            name='synoptictimeseries',
            unique_together=set([('synoptic_group_station', 'order'), ('synoptic_group_station', 'timeseries')]),
        ),
        migrations.AlterUniqueTogether(
            name='synopticgroupstation',
            unique_together=set([('synoptic_group', 'order')]),
        ),
    ]



from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^(?P<slug>[^/]+)/$', views.SynopticView.as_view(),
        name='synoptic_view'),
    url(r'^station/(?P<pk>\d+)/$', views.SynopticStationView.as_view(),
        name='synoptic_station_view'),
    url(r'^chart/(?P<pk>\d+)/$', views.ChartView.as_view(),
        name='synoptic_chart_view'),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enhydris_synoptic', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='synopticgroupstation',
            options={'ordering': ['synoptic_group', 'order']},
        ),
        migrations.AlterModelOptions(
            name='synoptictimeseries',
            options={'ordering': ['synoptic_group_station', 'order'], 'verbose_name_plural': 'Synoptic timeseries'},
        ),
    ]



from six import StringIO

from django import db
from django.conf import settings
from django.http import HttpResponse
from django.views.generic.base import View
from django.views.generic.detail import DetailView, SingleObjectMixin

import matplotlib; matplotlib.use('AGG')  # NOQA
from matplotlib.dates import DateFormatter, DayLocator, HourLocator
import matplotlib.pyplot as plt
import pandas as pd
from pthelma.timeseries import Timeseries

from enhydris_synoptic import models


def get_last_common_date(synoptic_timeseries):
    # We don't actually return the last common date, which would be
    # difficult; instead, we return the minimum of the last dates of the
    # timeseries, which will usually be the last common date. station is
    # an enhydris_synoptic.models.Station object.
    result = None
    for asynts in synoptic_timeseries:
        end_date = asynts.timeseries.end_date
        if end_date and ((not result) or (end_date < result)):
            result = end_date
    return result


def add_synoptic_group_station_context(synoptic_group_station):
    synoptic_timeseries = models.SynopticTimeseries.objects.filter(
        synoptic_group_station=synoptic_group_station)[:]
    synoptic_group_station.last_common_date = get_last_common_date(
        synoptic_timeseries)
    synoptic_group_station.synoptic_timeseries = []
    for asynts in synoptic_timeseries:
        synoptic_group_station.error = False
        tsrecords = Timeseries(asynts.timeseries.id)
        tsrecords.read_from_db(db.connection)
        try:
            asynts.value = tsrecords[synoptic_group_station.last_common_date]
        except KeyError:
            synoptic_group_station.error = True
            continue
        synoptic_group_station.synoptic_timeseries.append(asynts)


class SynopticView(DetailView):
    model = models.SynopticGroup
    slug_field = 'name'
    template_name = 'synopticgroup.html'

    def get_context_data(self, **kwargs):
        context = super(SynopticView, self).get_context_data(**kwargs)
        synoptic_group_stations = models.SynopticGroupStation.objects.filter(
            synoptic_group=self.object)[:]
        for synoptic_group_station in synoptic_group_stations:
            add_synoptic_group_station_context(synoptic_group_station)
        context['synoptic_group_stations'] = synoptic_group_stations
        return context


class SynopticStationView(DetailView):
    model = models.SynopticGroupStation
    template_name = 'synopticgroupstation.html'

    def get_context_data(self, **kwargs):
        context = super(SynopticStationView, self).get_context_data(**kwargs)
        add_synoptic_group_station_context(self.object)
        return context


class ChartView(SingleObjectMixin, View):
    model = models.SynopticTimeseries

    def get_color(self, i):
        """Return the color to be used for line with sequence i

        The first line to be drawn uses red, so self.get_color(0)='red';
        the second one uses green, so self.get_color(1)='green'; and there are
        also one or two more colors. We assume the user will not attempt to
        group more than a few time series together. However, if this happens,
        we recycle the colors starting from red again, to make sure we don't
        have an index error or anything.
        """
        colors = ['red', 'green', 'blue', 'magenta']
        return colors[i % len(colors)]

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()

        # Get all synoptic time series to put in the chart; i.e. the
        # requested one plus the ones that are grouped with it.
        synoptic_timeseries = [self.object]
        synoptic_timeseries.extend(models.SynopticTimeseries.objects.filter(
            group_with=self.object.id))

        # Read the last 144 records of each time series into a pandas object
        for t in synoptic_timeseries:
            ats = Timeseries(t.timeseries.id)
            ats.read_from_db(db.connection)
            buff = StringIO()
            ats.write(buff)
            buff.seek(0)
            t.pandas_object = pd.read_csv(buff, parse_dates=[0],
                                          usecols=[0, 1], index_col=0,
                                          header=None)[-144:]

        # Reorder them so that the greatest is at the top
        synoptic_timeseries.sort(key=lambda x: float(x.pandas_object.sum()),
                                 reverse=True)

        # Setup plot
        fig = plt.figure()
        fig.set_dpi(100)
        fig.set_size_inches(3.2, 2)
        fig.subplots_adjust(left=0.10, right=0.99, bottom=0.15, top=0.97)
        matplotlib.rcParams.update({'font.size': 7})
        ax = fig.add_subplot(1, 1, 1)

        # Draw lines. We use matplotlib's plot() instead of pandas's wrapper,
        # because otherwise there is trouble modifying the x axis labels
        # (see http://stackoverflow.com/questions/12945971/).
        for i, s in enumerate(synoptic_timeseries):
            xdata = s.pandas_object.index.to_pydatetime()
            ydata = s.pandas_object.values.transpose()[0]
            ax.plot(xdata, ydata, color=self.get_color(i),
                    label=s.get_subtitle())
            if i == 0:
                # We will need later the data of the first time series, in
                # order to fill the chart
                gydata = ydata

        # Change plot limits as needed
        xmin, xmax, ymin, ymax = ax.axis()
        if self.object.default_chart_min:
            ymin = min(self.object.default_chart_min, ymin)
        if self.object.default_chart_max:
            ymax = max(self.object.default_chart_max, ymax)
        ax.set_ylim([ymin, ymax])

        # Fill
        ax.fill_between(xdata, gydata, ymin, color='#ffff00')

        # X ticks and labels
        ax.xaxis.set_minor_locator(HourLocator(byhour=range(0, 24, 3)))
        ax.xaxis.set_minor_formatter(DateFormatter('%H:%M'))
        ax.xaxis.set_major_locator(DayLocator())
        ax.xaxis.set_major_formatter(
            DateFormatter('\n    %Y-%m-%d $\\rightarrow$'))

        # Gridlines and legend
        ax.grid(b=True, which='both', color='b', linestyle=':')
        if len(synoptic_timeseries) > 1:
            ax.legend()

        # Create plot
        result = HttpResponse(content_type="image/png")
        fig.savefig(result)

        # If unit testing, also return some data
        if hasattr(settings, 'TEST_MATPLOTLIB') and settings.TEST_MATPLOTLIB:
            data = [repr(line.get_xydata()).replace('\n', ' ')
                    for line in ax.lines]
            result['X-Matplotlib-Data'] = '(' + ', '.join(data) + ')'

        return result



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enhydris_synoptic', '0002_auto_20150813_1545'),
    ]

    operations = [
        migrations.AddField(
            model_name='synoptictimeseries',
            name='default_chart_max',
            field=models.FloatField(help_text='Maximum value of the y axis of the chart. If the variable goes lower, the chart will automatically expand. If empty, the chart will always expand just enough to accomodate the value.', null=True, blank=True),
        ),
        migrations.AddField(
            model_name='synoptictimeseries',
            name='default_chart_min',
            field=models.FloatField(help_text='Minimum value of the y axis of the chart. If the variable goes lower, the chart will automatically expand. If empty, the chart will always expand just enough to accomodate the value.', null=True, blank=True),
        ),
    ]



import six

from django.db import IntegrityError, models
from django.utils.translation import ugettext as _

from enhydris.hcore.models import Station, Timeseries


@six.python_2_unicode_compatible
class SynopticGroup(models.Model):
    name = models.SlugField(unique=True)
    stations = models.ManyToManyField(Station, through='SynopticGroupStation')

    def __str__(self):
        return self.name


@six.python_2_unicode_compatible
class SynopticGroupStation(models.Model):
    synoptic_group = models.ForeignKey(SynopticGroup)
    station = models.ForeignKey(Station)
    order = models.PositiveSmallIntegerField()
    timeseries = models.ManyToManyField(Timeseries,
                                        through='SynopticTimeseries')

    class Meta:
        unique_together = (('synoptic_group', 'order'),)
        ordering = ['synoptic_group', 'order']

    def __str__(self):
        return str(self.station)

    def check_timeseries_integrity(self, *args, **kwargs):
        """
        This method checks whether the timeseries.through.order field starts
        with 1 and is contiguous, and that groupped time series are in order.
        I wrote it thinking I could use it somewhere, but I don't think it's
        possible (see http://stackoverflow.com/questions/33500336/). However,
        since I wrote it, I keep it, although I'm not using it anywhere. Mind
        you, it's unit-tested.
        """
        # Check that time series are in order
        expected_order = 0
        for syn_ts in self.synoptictimeseries_set.order_by('order'):
            expected_order += 1
            if syn_ts.order != expected_order:
                raise IntegrityError(_(
                    "The order of the time series must start from 1 and be "
                    "continuous."))

        # Check that grouped time series are in order
        current_group_leader = None
        previous_synoptictimeseries = None
        for syn_ts in self.synoptictimeseries_set.order_by('order'):
            if not syn_ts.group_with:
                current_group_leader = None
                continue
            current_group_leader = current_group_leader or \
                previous_synoptictimeseries
            if syn_ts.group_with.id != current_group_leader.id:
                raise IntegrityError(_(
                    "Groupped time series must be ordered together."))
            previous_synoptictimeseries = syn_ts

        super(SynopticGroupStation, self).save(*args, **kwargs)


class SynopticTimeseriesManager(models.Manager):

    def primary(self):
        """Return only time series that don't have group_with."""
        return self.filter(group_with__isnull=True)


class SynopticTimeseries(models.Model):
    synoptic_group_station = models.ForeignKey(SynopticGroupStation)
    timeseries = models.ForeignKey(Timeseries)
    order = models.PositiveSmallIntegerField()
    title = models.CharField(max_length=50, blank=True, help_text=_(
        "Used as the chart title and as the time series title in the report. "
        "Leave empty to use the time series name."))
    group_with = models.ForeignKey('self', blank=True, null=True, help_text=_(
        "Specify this field if you want to group this time series with "
        "another in the same chart and in the report."))
    subtitle = models.CharField(max_length=50, blank=True, help_text=_(
        "If time series are grouped, this is shows in the legend of the chart "
        "and in the report, in brackets."))
    default_chart_min = models.FloatField(blank=True, null=True, help_text=_(
        "Minimum value of the y axis of the chart. If the variable goes "
        "lower, the chart will automatically expand. If empty, the chart will "
        "always expand just enough to accomodate the value."))
    default_chart_max = models.FloatField(blank=True, null=True, help_text=_(
        "Maximum value of the y axis of the chart. If the variable goes "
        "lower, the chart will automatically expand. If empty, the chart will "
        "always expand just enough to accomodate the value."))

    objects = SynopticTimeseriesManager()

    class Meta:
        unique_together = (('synoptic_group_station', 'timeseries'),
                           ('synoptic_group_station', 'order'),
                           )
        ordering = ['synoptic_group_station', 'order']
        verbose_name_plural = 'Synoptic timeseries'

    def __str__(self):
        return str(self.synoptic_group_station) + ' - ' + self.get_full_name()

    def get_title(self):
        return self.title or self.timeseries.name

    def get_subtitle(self):
        return self.subtitle or self.timeseries.name

    def get_full_name(self):
        result = self.get_title()
        if self.subtitle:
            result += ' (' + self.subtitle + ')'
        return result



#!/usr/bin/env python

from setuptools import setup, find_packages


installation_requirements = [
    "enhydris>=0.5,<0.7",
    "pandas>=0.14",
    "matplotlib>=1.4",
]

packages = find_packages(exclude=['*.tests', '*.tests.*', 'tests.*', 'tests',
                                  ])

kwargs = {
    'name': "enhydris-synoptic",
    'version': __import__('enhydris_synoptic').__version__,
    'license': "AGPL3",
    'description': "View current weather conditions in Enhydris",
    'author': "Antonis Christofides",
    'author_email': "anthony@itia.ntua.gr",
    'url': "https://github.com/openmeteo/enhydris-synoptic",
    'packages': packages,
    'install_requires': installation_requirements,
    'test_suite': 'runtests.runtests',
    'tests_require': ['model-mommy>=1.2.4'],
}

setup(**kwargs)
